//********************************************************************
//*** ResourcePool/SQL/SQLSelect.t.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.t.cc,v 1.26 2008/05/26 07:18:55 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"

using namespace std;
using namespace fatalmind;

class SQLSelect_test: public SQL::SQLFetcher {
	public:
		SQLSelect_test(int limit)
		: SQL::SQLFetcher()
		, _rows(0)
		, _limit(limit)
		{
		};

		virtual bool process(int row) {
			_rows = row;
			return row < _limit;
		}

		virtual void done(bool more, int fetched) {
			// the last processed row must match this one
			assertEqual(_rows, fetched);

			if (fetched < _limit) {
				assertFalse(more);
			}
			if (more) {
				// if there would have been more data, the limit must be reached
				assertEqual(_rows, _limit);
			} 
		}

		int _rows;
		int _limit;
};



void test_SQLSelect_init(RP& p)
{
	init(p);

	RP::Command::SQLExecute init(p, "insert into hirsch (id, ukn) values (?, ?)");
	init.bind<std::string>(1, "hohoho ' kasdfjs \" sdf");
	for (int i = 0; i < 600; ++i) {
		init.bind(0, i);
		p.execute(init);
	}
}


void test_SQLSelect_one(RP& p)
{
	string sql("select id, ukn from hirsch order by id");
	SQLSelect_test fetch(1000);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	p.execute(cmd);

	assertEqualT(int, col1, 599);
	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_SQLSelect_exception(RP& p)
{
	string sql("select id, ukn from hirsch blah blah blah order by id"); // invalid statement "blah blah blah" inside
	SQLSelect_test fetch(1000);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1 = 42;
	std::string col2 = "The deer";

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	try {
		p.execute(cmd);
		assertNotReach();
	} catch (...) {
	}

	assertEqualT(int, col1, 42);
	assertEqualT(std::string, col2, "The deer");
}

void test_perf_SQLSelect_all(RP& p)
{
	string sql("select id, ukn from hirsch");
	SQLSelect_test fetch(1000);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
		assertNotGrowing(i);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_ten(RP& p)
{
	string sql("select id, ukn from hirsch");
	SQLSelect_test fetch(10);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
		assertNotGrowing(i);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_one(RP& p)
{
	string sql("select id, ukn from hirsch where id = 356");
	SQLSelect_test fetch(1);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
		assertNotGrowing(i);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_SQLSelect_none(RP& p)
{
	string sql("select id, ukn from hirsch where id = ?");
	SQLSelect_test fetch(1000);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1 = 42;
	std::string col2 = "fourtytwo";

	cmd.bind(0, 31415926); // should not exist
	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
		assertNotGrowing(i);
	}
	assertEqual(col1, 42);
	assertEqual(col2, "fourtytwo");
}

void test_SQLSelect_clone(RP& p) {
	string sql("select id, ukn from hirsch where id = ?");
	SQLSelect_test fetch(10);
	RP::Command::SQLSelect cmd(p, sql, fetch);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	std::auto_ptr<RP::Command::SQLSelect> cmd2(cmd.clone<RP::Command::SQLSelect>());
	cmd.bind(0, 45); // available up to 599
	cmd2->bind(0, 46); // available up to 599

	p.execute(cmd);
	int x = fetch._rows;
	assertEqual(col1, 45);
	assertEqual(x, 1);
	assertEqual(fetch._rows, 1); // fetcher remains shared (will not be cloned)
	p.execute(*cmd2);
	assertEqual(col1, 46);
	assertEqual(x, 1);
	assertEqual(fetch._rows, 1); // fetcher was resetted
}

void test_SQLSelect_output(RP& p)
{
	SQLSelect_test fetch(10);
	RP::Command::SQLSelect cmd(p, "select id, ukn from hirsch where id = ?", fetch); // anyways not executed

	std::ostringstream ostr;
	ostr << cmd;
	std::string tmp;
	tmp = ostr.str();
	assertEqual(tmp.substr(tmp.find(":")), "::SQLSelect[select id, ukn from hirsch where id = ?|Bind[]]");
}

void test_SQLSelect_cleanup(RP& p) {
	drop(p);
}
